<?php
  // alerte.php

  // Connexion par HTTPS vers une machine distante pour envoyer les alertes
  // Completement autonome pour pouvoir etre utilise n'importe ou

  error_reporting(E_ALL | E_STRICT);

  // Cette section gère toutes les alertes qui sont envoyées par le cluster
  // vers la machine de supervision par HTTP(S)
  // machine : ip:port de la machine (utiliser l'IP en cas de panne DNS)
  $conf["alerte"]["machine"]="vigie.grenoble.cnrs.fr:443";
  // url d'appel : doit commencer par https:// ou http://
  $conf["alerte"]["url"]="https://vigie.grenoble.cnrs.fr/surveillance/".
                         "alerteAudio.php";
  // Temps maximum de connexion vers la station de supervision.
  // Le temps de lecture du message audio est compris dans ce temps !
  $conf["alerte"]["timeout"]=15;
  // Mail de l'administrateur en cas de problème avec la machine de supervision
  $conf["alerte"]["mail"]="CRIC@grenoble.cnrs.fr";

  // Lorsque le logiciel de cluster envoie des mails, il lui faut le serveur
  // sur lequel se connecter. On peut mettre un port spécial si necessaire avec
  // Adresse:NoPort, sinon le port 25 est utilise par defaut
  $conf["smtp_host"]="127.0.0.1";

  // Fournir ci-dessous l'adresse e-mail de l'expediteur
  $conf["smtp_from"]="alerte@vigie.grenoble.cnrs.fr";

  // Le logiciel peut utiliser une authentification pour envoyer le mail. Si
  // vous ne souhaitez pas utiliser cette authentification, mettre ""
  $conf["smtp_user"]="";
  $conf["smtp_pass"]="";

  $conf["repRun"] = "/tmp";


  // Le fichier de log local doit etre passe en parametre :
  // $conf["loggerFile"] = "/var/log/XXX.log";

  ////////////////////////////////
  // VERIFICATIONS DE DEMARRAGE //
  ////////////////////////////////
  if (!in_array("https",stream_get_wrappers()))
    die ("Pas de support HTTPS activé pour le PHP\n");
  if (!isset($conf["alerte"]["url"]))
    die ("\$conf[alerte][url] est indéfinie\n");
  if (!isset($conf["alerte"]["machine"]))
    die ("\$conf[alerte][machine] est indéfinie\n");
  if (!isset($conf["alerte"]["timeout"]))
    die ("\$conf[alerte][timeout] est indéfinie\n");
  if ($conf["alerte"]["timeout"]<1)
    die ("\$conf[alerte][timeout] est inférieure a 1\n");
  if (!isset($conf["alerte"]["mail"]))
    die ("\$conf[alerte][mail] est indéfinie");

  ////////////////////
  // CONFIRGURATION //
  ////////////////////

  ///////////////
  // FONCTIONS //
  ///////////////
  function hostnameLocal()
  {
    // Renvoie le FQDN de la machine locale
    return trim(file_get_contents("/etc/hostname"));
  }

  function hostname($fqdn)
  {
    // Renvoie le nom de la machine dans le $fqdn passe
    $pos=strpos($fqdn,".");
    // Si il n'y a pas de . dans le nom (c'est deja un hostname), renvoie le
    // $fqdn passe
    if ($pos===FALSE) return $fqdn;
    // OK : on renvoie le nom
    return substr($fqdn,0,$pos);
  }

  function criclog($msg)
  {
    // Gestionnaire de messages d'erreur (si on est en mode trigger_error, on
    // fourre d'office l'erreur dans le trigger_error
    global $conf, $argv;
    $fp=fopen ($conf["loggerFile"],"a");
    if ($fp===FALSE)
      die ("Le fichier de log ".$conf["loggerFile"]." n'est pas "
          ."inscriptible\n");
    $date=date("d/m/Y H:i:s");
    $serveur=hostname(hostnameLocal());
    $progname=substr(basename($argv[0]),0,-4);
    if (gettype($msg)=="array")
    {
      foreach ($msg as $key=>$ligne)
      {
        fwrite ($fp, $date." [".$serveur."] [".sprintf("%18s",$progname).
                     "] ".$key."=>".$ligne."\n");
      }
    }
    else
    {
      fwrite ($fp, $date." [".$serveur."] [".sprintf("%18s",$progname).
                   "] ".$msg."\n");
    }
    fflush($fp);
    fclose ($fp);
  }

  function smtp_mail($to, $subject, $message, $headers = "")
  {
    global $conf;

    $recipients = explode(",", $to);

    // Are we using port 25 or a custom port?
    if (strpos($conf["smtp_host"], ":") !== false)
    {
      list($smtp_host, $smtp_port) = explode(":", $conf["smtp_host"]);
    }
    else
    {
      $smtp_host = $conf["smtp_host"];
      $smtp_port = 25;
    }

    if (!($socket = @fsockopen($smtp_host, $smtp_port, $errno, $errstr, 5)))
    {
      criclog("Could not connect to smtp host '".$conf["smtp_host"]."' (".
              $errno.") (".$errstr.")", __FILE__, __LINE__);
      return FALSE;
    }
    stream_set_timeout  ($socket, 5);

    if (!server_parse($socket, "220")) return;

    if ($conf["smtp_user"] != "" && $conf["smtp_pass"] != "")
    {
      fwrite($socket, "EHLO ".$smtp_host."\r\n");
      if (!server_parse($socket, "250")) return;

      fwrite($socket, "AUTH LOGIN"."\r\n");
      if (!server_parse($socket, "334")) return;

      fwrite($socket, base64_encode($conf["smtp_user"])."\r\n");
      if (!server_parse($socket, "334")) return;

      fwrite($socket, base64_encode($conf["smtp_pass"])."\r\n");
      if (!server_parse($socket, "235")) return;
    }
    else
    {
      fwrite($socket, "HELO ".$smtp_host."\r\n");
      if (!server_parse($socket, "250")) return;
    }

    fwrite($socket, "MAIL FROM: <".$conf["smtp_from"].">\r\n");
    if (!server_parse($socket, "250")) return;

    $to_header = "To: ";

    @reset($recipients);
    $i = 0;
    foreach($recipients as $email)
    {
      fwrite($socket, "RCPT TO: <".$email.">"."\r\n");
      if (!server_parse($socket, "250")) return;
      if ($i > 0) $to_header .= ", ";
      $to_header .= "<".$email.">";
      $i++;
    }
    fwrite($socket, "DATA"."\r\n");
    if (!server_parse($socket, "354")) return;

    fwrite($socket, "Subject: ".$subject."\r\n".$to_header."\r\n".$headers.
           "\r\n\r\n".$message."\r\n");

    fwrite($socket, "."."\r\n");
    if (!server_parse($socket, "250")) return;

    fwrite($socket, "QUIT"."\r\n");
    fclose($socket);

    return true;
  }

  function server_parse($socket, $expected_response)
  {
    $server_response = "";
    $startTime = microtime(TRUE);
    while ($startTime+2.00 > microtime(TRUE) &&
           substr($server_response, 3, 1) != " ")
    {
      if (!($server_response = fgets($socket, 256)))
      {
        // J'ai vire ce message inutile qui boucle si le serveur tombe pendant
        // une transaction, remplace par le message ci-
        // dessous "Serveur SMTP HS : dehors"
    //  criclog("Couldn't get mail server response codes. Please contact the ".
    //          "administrator.", __FILE__, __LINE__);
      }
    }
    if ($server_response === FALSE)
    {
      criclog ("Serveur SMTP HS pendant la communication : dehors");
      return FALSE;
    }

    if (!(substr($server_response, 0, 3) == $expected_response))
    {
      criclog("Unable to send e-mail. Please contact the administrator with ".
              "the following error message reported by the SMTP server: '".
              $server_response."'", __FILE__, __LINE__);
      return FALSE;
    }
    return TRUE;
  }

  function alerte ($audio, $finAudio="")
  {
    // Cette fonction envoie une connexion HTTPS vers un serveur defini dans la
    // configuration avec les informations contenues dans les parametres.
    // En cas de panne de la station de supervision, un mail est envoye a
    // l'administrateur defini dans la configuration.
    // $audio est la serie de son joues
    // $finAudio est le message oppose au message audio actuel. Utilise pour
    //   detecter la fin d'un probleme et remettre le son en fonction apres une
    //   demande de mute

    global $conf;
    $url=parse_url($conf["alerte"]["url"]);
    if ($url["scheme"]==="https")
      $protocole="ssl";
    elseif ($url["scheme"]==="http")
      $protocole="tcp";
    else
    {
      criclog ("alerte.php : schéma de l'URL de \$conf[alerte][url] inconnu");
      return;
    }
    unset ($erreur);
    unset ($erreurnet);
    unset ($GLOBALS["erreurnet"]);
    $old_error_handler = set_error_handler(
      function ($errno, $errstr, $errfile, $errline)
      {
        global $erreurnet;
        $GLOBALS["erreurnet"] .= $errstr."\n";
        return true;
      });
    $fp=@stream_socket_client("$protocole://".$conf["alerte"]["machine"],
                              $errno, $errstr, $conf["alerte"]["timeout"]);
    restore_error_handler();
    if (!$fp)
    {
      // La machine de supervision est injoignable
      $erreurnet=$errstr;
      if (isset ($GLOBALS["erreurnet"]))
        $erreurnet .= rtrim ($GLOBALS["erreurnet"]);
      //echo "$errstr ($errno)\n";
    }
    elseif (trim ($audio) === "")
    {
      $erreur = "Message audio vide envoyé a alerte()";
    }
    else
    {
      // Si le serveur met plus de $timeout secondes a repondre, il y a un
      // probleme.
      stream_set_timeout ($fp, $conf["alerte"]["timeout"]);
      $finAudio = trim ($finAudio);

      $serveur=hostname(hostnameLocal());
      $commande ="";
      if ($audio !== "test")
        $audio="$serveur : $audio";
      $commande.="audio=".urlencode($audio);
      if (isset ($finAudio) && $finAudio!=="")
      {
        $finAudio="$serveur : $finAudio";
        $commande.="&finAudio=".urlencode($finAudio);
      }

      $headers ="POST ".$url["path"]." HTTP/1.1\r\n";
      $headers.="Host: ".$url["host"]."\r\n";
      $headers.="User-Agent: CRIC-Alerte\r\n";
      $headers.="Content-Length: ".strlen($commande)."\r\n";
      $headers.="Content-Type: application/x-www-form-urlencoded\r\n";

      $transaction =$headers;
      $transaction.="\r\n";
      $transaction.=$commande;
      // echo $transaction;

      fwrite ($fp, $transaction);
      $result=FALSE;
      // On veut le code de retour du serveur
      // Il est de la forme :
      // HTTP/1.1 404 Not Found
      $result=fgets ($fp, 1024);
      // On recupere  le code et le message
      $arr = explode (" ", $result);
      $http = $arr[0];
      if ($result === false)
        $result = "bool FALSE";
      if (! array_key_exists (1, $arr))
      {
        $erreur = "Pas de code fourni par le serveur : '$result'\n";
        $code = 500;
      }
      else
      {
        $code = $arr[1];
      }
      if (! array_key_exists (2, $arr))
      {
        $erreur = "Pas de message fourni par le serveur : '$result'\n";
        $code = 500;
      }
      else
      {
        $message = $arr[2];
      }
      switch ($code)
      {
        case "404":
          $erreur="404 URL introuvable....\n".$conf["alerte"]["url"]."\n";
          break;
        case "200":
           // Cool la connexion marche bien, regarde si on a eu une erreur
           break;
        default:
          // Un erreur est signalee, on va envoyer un mail
          $erreur=$result;
          break;
      }
      if (!isset($erreur))
      {
        // On saute les entetes du serveur (jusqu'a la ligne blanche)
        while (($ligne=fgets($fp, 1024))!==FALSE)
        {
          if (trim($ligne)==="") break;
        }
        $result="";
        while (($ligne=fgets($fp, 1024))!==FALSE)
        {
          $result.=$ligne;
        }
        // On verifie si l'arret est du a une fin de connexion a cause de
        // timeout
        $info = stream_get_meta_data($fp);
        if ($info['timed_out'])
        {
          $erreurnet="Connection timed out !";
        }
        if ($result!=="OK\n")
        {
          // Une erreur a ete renvoyee par le PHP de la supervision, on envoie
          // un mail
          $erreur=$result;
        }
      }
      fclose($fp);
    }
    if (isset ($erreur))
    {
      // Erreur signalee lors de la connexion avec le serveur de surveillance
      // on envoie un mail a l'administrateur
      $sujet="ALERTE : Problème d'alerte sur ".hostnameLocal();
      $message ="Message du gestionnaire d'alerte de ".hostnameLocal()." :\r\n";
      $message.="\r\n";
      $message.="La machine de supervision '".$conf["alerte"]["machine"];
      $message.="' a renvoyé l'erreur suivante :\r\n";
      $message.=$erreur;
      criclog ("ERREUR : '".$conf["alerte"]["machine"]."' : ".trim($erreur));
    }
    if (isset ($erreurnet))
    {
      // Erreur signalee lors de la connexion avec le serveur de surveillance
      // on envoie un mail a l'administrateur
      $sujet="ALERTE : Problème d'alerte sur ".hostnameLocal();
      $message ="Message du gestionnaire d'alerte de ".hostnameLocal()." :\r\n";
      $message.="\r\n";
      $message.="La machine de supervision '".$conf["alerte"]["machine"];
      $message.="' est injoignable : \r\n";
      $message.=$erreurnet;
      criclog ("ERREUR : '".$conf["alerte"]["machine"]."' injoignable : ".
               trim($erreurnet));
    }
    if (isset ($erreur) || isset ($erreurnet))
    {
      // Une erreur reseau ou une erreur de fonctionnement a ete detectee, on
      // envoie le mail avec le message et le sujet definis ci-dessus, sauf si
      // le meme mail a deja ete envoye dans les 30 dernieres minutes.
      $fichierBloqueMail=$conf["repRun"]."/".urlencode($sujet);
      clearstatcache();
      if (!file_exists($fichierBloqueMail) ||
          filemtime($fichierBloqueMail)+60*30<time())
      {
        touch ($fichierBloqueMail);
        $headers='From: CRIC-Alerte@'.hostnameLocal() . "\r\n" .
                 'Reply-To: '.$conf["alerte"]["mail"]. "\r\n" .
                 'X-Mailer: PHP/' . phpversion()."\r\n".
                 'Content-Type: text/plain; charset=utf-8';
        /*
        echo "*********** MAIL ************\n";
        echo $headers."\n";
        echo $sujet."\n".$message;
        echo "*****************************\n";
        return FALSE;
        */
        if (smtp_mail ($conf["alerte"]["mail"], $sujet, $message, $headers)===
            FALSE)
        {
          criclog ("ERREUR : Le mail d'information n'est pas parti");
        }
      }
      return FALSE;
    }
    return $result;
  }

  ///////////
  // TESTS //
  ///////////
  /*
  echo alerte ("test");
  echo alerte ("Message de test 1");
  echo alerte ("Message de test 2", "coucou behh");
  echo alerte ("Message de test 3", "coucou", 2)
  echo alerte ("demarrageMachine");
  */
